@@ -1,6 +1,5 @@
 """This module contains vpype core and its API."""
 
-from ._deprecated import *
 from .config import *
 from .filters import *
 from .geometry import *
@@ -14,9 +13,9 @@ from .utils import *
 
 
 def _get_version() -> str:
-    import pkg_resources
+    from importlib.metadata import version
 
-    return pkg_resources.get_distribution("vpype").version
+    return version(__name__)
 
 
 __version__ = _get_version()
