@@ -1,37 +1,54 @@
+from __future__ import annotations
+
 import math
 import re
-from typing import Callable, Dict, List, Tuple, Union
+from typing import Callable
 
 import numpy as np
 
 # REMINDER: anything added here must be added to docs/api.rst
 __all__ = [
     "UNITS",
+    "UNIT_SYSTEMS",
     "ANGLE_UNITS",
     "PAGE_SIZES",
     "convert_length",
+    "format_length",
     "convert_angle",
     "convert_page_size",
     "union",
 ]
 
 
-def _mm_to_px(x: float, y: float) -> Tuple[float, float]:
+def _mm_to_px(x: float, y: float) -> tuple[float, float]:
     return x * 96.0 / 25.4, y * 96.0 / 25.4
 
 
+# NOTE: order matters due to the implementation of _convert_unit()
+
 UNITS = {
     "px": 1.0,
     "in": 96.0,
+    "inch": 96.0,
+    "ft": 12.0 * 96.0,
+    "yd": 36.0 * 96.0,
+    "mi": 1760.0 * 36.0 * 96.0,
     "mm": 96.0 / 25.4,
     "cm": 96.0 / 2.54,
+    "m": 100.0 * 96.0 / 2.54,
+    "km": 100_000.0 * 96.0 / 2.54,
     "pc": 16.0,
     "pt": 96.0 / 72.0,
 }
 
+UNIT_SYSTEMS = {
+    "metric": ["mm", "cm", "m", "km"],
+    "imperial": ["in", "ft", "yd", "mi"],
+}
+
 ANGLE_UNITS = {
     "deg": 1.0,
-    "grad": 9.0 / 10.0,  # note: must be before "rad"!
+    "grad": 9.0 / 10.0,
     "rad": 180.0 / math.pi,
     "turn": 360.0,
 }
@@ -52,20 +69,61 @@ PAGE_SIZES = {
     "tabloid": _mm_to_px(279.4, 431.8),
 }
 
+_FLOAT_WITH_UNIT_RE = re.compile(r"^([+\-0-9.e]*)([a-z]*)$", flags=re.IGNORECASE)
 
-def _convert_unit(value: Union[str, float], units: Dict[str, float]) -> float:
+
+def _convert_unit(value: str | float | int, units: dict[str, float]) -> float:
     """Converts a string with unit to a value"""
-    if isinstance(value, str):
-        value = value.strip().lower()
-        for unit, factor in units.items():
-            if value.endswith(unit):
-                num = value.strip(unit)
-                return (float(num) if len(num) > 0 else 1.0) * factor
+    if isinstance(value, (float, int)):
+        return value
 
-    return float(value)
+    mo = _FLOAT_WITH_UNIT_RE.match(value.strip().lower())
+    try:
+        if mo is None or value == "":
+            raise ValueError
+        number = mo.groups()[0]
+        unit = mo.groups()[1]
+        return (float(number) if number else 1.0) * (units[unit] if unit else 1.0)
+    except (ValueError, KeyError):
+        raise ValueError(f"cannot convert value '{value}'")
 
 
-def convert_length(value: Union[str, float]) -> float:
+def format_length(value: float, unit: str, full_precision: bool = True) -> str:
+    """Convert a length in pixel to a string representation with the provided unit or unit
+    system.
+
+    The representation is for display only and
+
+    Args:
+        value: the value to convert (in pixels)
+        unit: unit (from :data:`vpype.UNITS`) or unit system (from :data:`vp.UNIT_SYSTEMS`)
+        full_precision: if False, the return string will use a limited precision best suited
+            for informative display
+
+    Returns:
+        a string representation of the length
+    """
+    if unit in UNITS:
+        units = [unit]
+    elif unit in UNIT_SYSTEMS:
+        units = UNIT_SYSTEMS[unit]
+    else:
+        raise ValueError(f"unknown unit '{unit}'")
+
+    scaled_values = zip((value / UNITS[unit] for unit in units), units)
+    picked_value, picked_unit = sorted(
+        scaled_values, key=lambda x: abs(math.log10(abs(x[0])) - 0.5) if abs(x[0]) > 0 else 0
+    )[0]
+
+    if full_precision:
+        return str(picked_value) + picked_unit
+    elif picked_value < 0.1:
+        return f"{picked_value:.4g}" + picked_unit
+    else:
+        return f"{picked_value:.3f}" + picked_unit
+
+
+def convert_length(value: str | float) -> float:
     """Convert a length optionally expressed as a string with unit to px value.
 
     Args:
@@ -80,7 +138,7 @@ def convert_length(value: Union[str, float]) -> float:
     return _convert_unit(value, UNITS)
 
 
-def convert_angle(value: Union[str, float]) -> float:
+def convert_angle(value: str | float) -> float:
     """Convert an angle optionally expressed as a string with unit to degrees.
 
     Args:
@@ -95,7 +153,7 @@ def convert_angle(value: Union[str, float]) -> float:
     return _convert_unit(value, ANGLE_UNITS)
 
 
-def convert_page_size(value: str) -> Tuple[float, float]:
+def convert_page_size(value: str) -> tuple[float, float]:
     """Converts a string with page size to dimension in pixels.
 
     The input can be either a known page size (see ``vpype write --help`` for a list) or
@@ -145,7 +203,7 @@ def convert_page_size(value: str) -> Tuple[float, float]:
     return float(x) * convert_length(x_unit), float(y) * convert_length(y_unit)
 
 
-def union(line: np.ndarray, keys: List[Callable[[np.ndarray], bool]]) -> bool:
+def union(line: np.ndarray, keys: list[Callable[[np.ndarray], bool]]) -> bool:
     """Returns True if every callables in ``keys`` return True (similar to ``all()``. This
     function is typically used with :meth:`LineCollection.filter`.
 
