@@ -1,5 +1,6 @@
+from __future__ import annotations
+
 import random
-from typing import Optional
 
 import click
 
@@ -23,7 +24,7 @@ __all__ = ("lcopy", "lmove", "ldelete", "lreverse", "lswap")
 )
 @click.option("-m", "--no-prop", is_flag=True, help="Do not copy metadata.")
 @global_processor
-def lcopy(document, sources, dest, prob: Optional[float], no_prop: bool):
+def lcopy(document, sources, dest, prob: float | None, no_prop: bool):
     """Copy the content of one or more layer(s) to another layer.
 
     SOURCES can be a single layer ID, the string 'all' (to copy all non-empty layers,
@@ -93,7 +94,7 @@ content is not duplicated:
 )
 @click.option("-m", "--no-prop", is_flag=True, help="Do not move metadata.")
 @global_processor
-def lmove(document, sources, dest, prob: Optional[float], no_prop: bool):
+def lmove(document, sources, dest, prob: float | None, no_prop: bool):
     """Move the content of one or more layer(s) to another layer.
 
     SOURCES can be a single layer ID, the string 'all' (to copy all non-empty layers,
@@ -125,8 +126,10 @@ def lmove(document, sources, dest, prob: Optional[float], no_prop: bool):
     src_lids = multiple_to_layer_ids(sources, document)
     dest_lid = single_to_layer_id(dest, document)
 
-    if dest_lid in src_lids:
-        src_lids.remove(dest_lid)
+    if dest_lid in document.layers:
+        dest_lc = document.layers[dest_lid].clone()
+    else:
+        dest_lc = vp.LineCollection()
 
     move_metadata = len(src_lids) == 1 and prob is None and not no_prop
     source_metadata = document.layers[src_lids[0]].metadata if move_metadata else {}
@@ -148,12 +151,14 @@ def lmove(document, sources, dest, prob: Optional[float], no_prop: bool):
                 document.pop(lid)
 
             if len(moving_lines) > 0:
-                document.add(moving_lines, dest_lid)
+                dest_lc.extend(moving_lines)
         else:
-            document.add(document.pop(lid), dest_lid)
+            dest_lc.extend(document.pop(lid))
             if move_metadata:
-                document.layers[dest_lid].metadata.update(source_metadata)
+                dest_lc.metadata.update(source_metadata)
 
+    if len(dest_lc) > 0:
+        document.add(dest_lc, dest_lid, with_metadata=True)
     return document
 
 
@@ -169,7 +174,7 @@ def lmove(document, sources, dest, prob: Optional[float], no_prop: bool):
     help="Path deletion probability (default: 1.0).",
 )
 @global_processor
-def ldelete(document: vp.Document, layers, keep: bool, prob: Optional[float]) -> vp.Document:
+def ldelete(document: vp.Document, layers, keep: bool, prob: float | None) -> vp.Document:
     """Delete one or more layers.
 
     LAYERS can be a single layer ID, the string 'all' (to delete all layers), or a
@@ -216,7 +221,7 @@ def ldelete(document: vp.Document, layers, keep: bool, prob: Optional[float]) ->
 @click.option("-m", "--no-prop", is_flag=True, help="Do not move metadata.")
 @global_processor
 def lswap(
-    document: vp.Document, first: int, second: int, prob: Optional[float], no_prop: bool
+    document: vp.Document, first: int, second: int, prob: float | None, no_prop: bool
 ) -> vp.Document:
     """Swap the content between two layers
 
