@@ -2,12 +2,14 @@ from __future__ import annotations
 
 import logging
 import math
-from typing import cast
+from typing import List, Optional, Union, cast
 
 import click
 import numpy as np
 
 import vpype as vp
+from vpype.geometry import line_length
+from vpype.model import LineCollection
 
 from .cli import cli
 from .decorators import global_processor, layer_processor
@@ -27,6 +29,7 @@ __all__ = (
     "reverse",
     "snap",
     "splitall",
+    "splitdist",
     "trim",
 )
 
@@ -509,6 +512,8 @@ SIZE may be one of:
 
     {', '.join(vp.PAGE_SIZES.keys())}
 
+Note that `tight` is special case, see below.
+
 Alternatively, a custom size can be specified in the form of WIDTHxHEIGHT. WIDTH and
 HEIGHT may include units. If only one has an unit, the other is assumed to have the
 same unit. If none have units, both are assumed to be pixels by default. Here are some
@@ -528,13 +533,24 @@ alignment can be adjusted using the `--align`, resp. `--valign` options.
 Optionally, this command can scale the geometries to fit specified margins with the
 `--fit-to-margins` option.
 
+When SIZE is `tight`, the page size is set to fit the width and height of the existing
+geometries. If `--fit-to-margins` is used, the page size is increased to accommodate the
+margin. By construction, `--align` and `--valign` have no effect with `tight`.
+
+On an empty pipeline, `layout` simply sets the page size to SIZE, unless `tight` is used. In
+this case, `layout` has no effect at all.   
+
 Examples:
 
 \b
     Fit the geometries to 3cm margins with top alignment (a generally
     pleasing arrangement for square designs on portrait-oriented pages):
 
-        vpype read input.svg layout --fit-to-margins 3cm --valign top a4 write.svg
+        vpype read input.svg layout --fit-to-margins 3cm --valign top a4 write output.svg
+        
+    Set the page size to the geometries' boundary, with a 1cm margin:
+    
+        vpype read input.svg layout --fit-to-margins 3cm tight write output.svg
 """
 
 
@@ -575,17 +591,26 @@ def layout(
     """Layout command"""
 
     size = _normalize_page_size(size, landscape)
-
-    document.page_size = size
     bounds = document.bounds()
+    tight = size == vp.PAGE_SIZES["tight"]
 
+    # handle empty geometry special cases
     if bounds is None:
-        # nothing to layout
+        if not tight:
+            document.page_size = size
         return document
 
     min_x, min_y, max_x, max_y = bounds
     width = max_x - min_x
     height = max_y - min_y
+
+    # handle "tight" special case
+    if tight:
+        extra = 2 * (margin or 0.0)
+        size = width + extra, height + extra
+
+    document.page_size = size
+
     if margin is not None:
         document.translate(-min_x, -min_y)
         scale = min((size[0] - 2 * margin) / width, (size[1] - 2 * margin) / height)
@@ -690,3 +715,52 @@ def reverse(line_collection: vp.LineCollection) -> vp.LineCollection:
 
     line_collection.reverse()
     return line_collection
+
+
+@cli.command(group="Operations")
+@click.argument("dist", type=LengthType(), required=True)
+@click.option(
+    "-l",
+    "--layer",
+    type=LayerType(accept_multiple=True),
+    default="all",
+    help="Target layer(s).",
+)
+@global_processor
+def splitdist(
+    document: vp.Document, dist: float, layer: int | list[int] | None
+) -> vp.Document:
+    """Split layers by drawing distance.
+
+    This command will keep the N first lines whose cumulated length is less than DIST.
+    The remaining lines are spread to the next available layers.
+
+    This command works at the line level. A finer-grained result can be obtained at the segment
+    level using the `splitall splitdist linemerge` sequence of commands.
+    """
+    new_doc = document.clone(keep_layers=True)
+    layer_ids = multiple_to_layer_ids(layer, document)
+
+    for layer_id, lines in document.layers.items():
+        if layer_id not in layer_ids:
+            new_doc.add(lines, layer_id)
+            continue
+
+        cumulative_length = 0
+        split_lines = lines.clone()
+        num_lines = len(lines)
+
+        for i, line in enumerate(lines):
+            current_line_length = np.sum(np.abs(np.diff(line)))
+            split_lines.append(line)
+            cumulative_length += current_line_length
+
+            if cumulative_length >= dist or i == num_lines - 1:
+                if new_doc.layers[layer_id].is_empty():
+                    new_doc.add(split_lines, layer_id, with_metadata=True)
+                else:
+                    new_doc.add(split_lines, new_doc.free_id(), with_metadata=True)
+                split_lines = lines.clone()
+                cumulative_length = 0
+
+    return new_doc
